// CORBAץ
// client for mico

#include "../config.h"

#include <stdio.h>
#include <iostream>
using namespace std;

#include <CORBA.h>
#ifdef USE_NAME_SERVICE
  #include <mico/CosNaming.h>
#endif

#include "counter.h"
#include "../misc.h"

////////////////////////////////////////////////////////////////////////

void do_test(Counter_ptr counter)
{
    CORBA::String_var name = counter->name();
    printf("name = %s\n", name.in());

    int i;
    printf("incr(): ");
    for (i = 0; i < 5; i++) {
        counter->incr();
        printf(" %d", counter->value());
    }
    printf(".\n");

    printf("decr(): ");
    for (i = 0; i < 3; i++) {
        counter->decr();
        printf(" %d", counter->value());
    }
    printf(".\n");
}

#ifndef USE_NAME_SERVICE
const char* ior_file = "../counter.ior";
#endif

int main(int argc, char* argv[])
{
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "mico-local-orb");

    Counter_var nc;
    ReverseCounter_var rc;

#ifdef USE_NAME_SERVICE
    CORBA::Object_var nobj = orb->resolve_initial_references("NameService");
    if (!nobj) {
        printf("resolve NameService failed.\n");
        return 1;
    }
    CosNaming::NamingContext_var naming = CosNaming::NamingContext::_narrow(nobj);

    CosNaming::Name name;
    name.length(1);
    name[0].id = CORBA::string_dup(COUNTER_NAME);
    name[0].kind = CORBA::string_dup("");
    CORBA::Object_var no = naming->resolve(name);
    nc = Counter::_narrow(no);

    name.length(1);
    name[0].id = CORBA::string_dup(REVERSE_NAME);
    name[0].kind = CORBA::string_dup("");
    CORBA::Object_var ro = naming->resolve(name);
    rc = ReverseCounter::_narrow(ro);
#else
    FILE* fp = fopen(ior_file, "r");
    if (!fp) {
        perror("file open");
        return 1;
    }
    try {
        CORBA::Object_var obj_nc = orb->string_to_object(read_line(fp));
        CORBA::Object_var obj_rc = orb->string_to_object(read_line(fp));
        fclose(fp);
        if (!obj_nc || !obj_rc) {
            printf("ior error.\n");
            return 1;
        }

        nc = Counter::_narrow(obj_nc);
        rc = ReverseCounter::_narrow(obj_rc);
    }
    catch (CORBA::Exception& e) {
        e._print(cout);
    }
    catch (...) {
        printf("unknown exception.\n");
    }
#endif  // USE_NAME_SERVICE

    if (CORBA::is_nil(nc) || CORBA::is_nil(rc)) {
        printf("cannot invoke on a nil object\n");
        return 1;
    }

    do_test(nc);
    do_test(rc);

    return 0;
}
