// CORBAץ
// client for ORBit

#include "../config.h"

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <orbit/orbit.h>

#ifdef USE_NAME_SERVICE
  #include <ORBitservices/CosNaming.h>
#endif

#include "../misc.h"

#include "counter.h"
    // orbit-idlˤä줿.hե
    // counter.idl -> counter.h

//////////////////////////////////////////////////////////////////////

CORBA_Environment ev;

void testCounter(Counter counter) {
    printf("name = %s\n", Counter_name(counter, &ev));

    int i;
    printf("incr(): ");
    for (i = 0; i < 10; i++) {
        Counter_incr(counter, &ev);
        printf(" %d", Counter__get_value(counter, &ev));
    }
    printf(".\n");

    printf("decr(): ");
    for (i = 0; i < 3; i++) {
        Counter_decr(counter, &ev);
        printf(" %d", Counter__get_value(counter, &ev));
    }
    printf(".\n");
}

void catchException(CORBA_Environment& ev) {
    if (ev._major != CORBA_NO_EXCEPTION) {
        printf("CORBA exception: %s\n", CORBA_exception_id(&ev));
        CORBA_exception_free(&ev);
        exit(1);
    }
}

#ifdef USE_NAME_SERVICE
void get_object_by_nameservice(CORBA_ORB orb, Counter& nc, ReverseCounter& rc) {
    CosNaming_NamingContext naming;
#if 0
    // ޥɥ饤ǥ͡ॵСIORͿ
    naming = CORBA_ORB_resolve_initial_references(orb, "NameService", &ev);
#else
    // 2000.11.01 ORBitstring_to_object()ǤǤ
    FILE* fp = fopen("../naming.ior", "r");
    assert(fp);
    naming = CORBA_ORB_string_to_object(orb, read_line(fp), &ev);
    fclose(fp);
#endif
    catchException(ev);
    if (!naming) {
        printf("resolve NameService failed.\n");
        return;
    }

    CosNaming_Name* name = CosNaming_Name__alloc();
    name->_length = 1;
    name->_buffer = CORBA_sequence_CosNaming_NameComponent_allocbuf(1);
    name->_buffer->id = CORBA_string_dup(COUNTER_NAME);
    name->_buffer->kind = CORBA_string_dup("");
    nc = CosNaming_NamingContext_resolve(naming, name, &ev);

    name = CosNaming_Name__alloc();
    name->_length = 1;
    name->_buffer = CORBA_sequence_CosNaming_NameComponent_allocbuf(1);
    name->_buffer->id = CORBA_string_dup(REVERSE_NAME);
    name->_buffer->kind = CORBA_string_dup("");
    rc = CosNaming_NamingContext_resolve(naming, name, &ev);
}
#else // USE_NAME_SERVICE

const char* ior_file = "../counter.ior";

void get_object_by_ior(CORBA_ORB orb, Counter& nc, ReverseCounter& rc) {
    FILE* fp = fopen(ior_file, "r");
    if (!fp) {
        perror("ior-file open");
        return;
    }
    nc = CORBA_ORB_string_to_object(orb, read_line(fp), &ev);
    catchException(ev);
    rc = CORBA_ORB_string_to_object(orb, read_line(fp), &ev);
    catchException(ev);
    fclose(fp);
}
#endif // USE_NAME_SERVICE

int main(int argc, char* argv[])
{
    CORBA_exception_init(&ev);

    CORBA_ORB orb = CORBA_ORB_init(&argc, argv, "orbit-local-orb", &ev);
    catchException(ev);

    Counter counter = NULL;
    ReverseCounter rc = NULL;

#ifdef USE_NAME_SERVICE
    get_object_by_nameservice(orb, counter, rc);
#else
    get_object_by_ior(orb, counter, rc);
#endif

    if (!counter || !rc) {
        printf("ior error.\n");
        return 1;
    }

    testCounter(counter);
    testCounter(rc);

    CORBA_Object_release(counter, &ev);
    CORBA_Object_release(rc, &ev);

    return 0;
}
